/***********************************************************************************
    Copyright 2011 Bryan Johnson, Xiaoyue Guan & Charles F. Gammie

                  mthreed  version 1.0   (released Jun 2, 2011)

    This file is part of mthreed. mthreed adopts a ZEUS-like scheme to solve 
    magnetohydrodynamic equations of motion in a shearing sheet. It uses obital 
    advection to ensure fast and accurate integration in a large shearing box. 
    

    You are morally obligated to cite the following paper in his/her
    scientific literature that results from use of any part of mthreed:

    [1] Johnson, B.M., Guan, X. \& Gammie, C.F. \ 2008, Astrophysical Journal, 
        Supplement, 177, 373


    The latest version of the mthreed is available on our code distribution
    website:
    http://rainman.astro.uiuc.edu/codelib/

    mthreed is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    mthreed is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mthreed; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

***********************************************************************************/

/* enforce boundary conditions */

#define	REFLECTING	0
#define PERIODIC	1

#include "decs.h"

void bound_var(REAL (*var)[NY+5][NZ+5], REAL tcurr, int sym)
{
	int i,j,k,lo ;
	REAL del ;
        REAL test;
	
	if(var==rho) lo = -3 ;
	else lo = -2 ;

	/* x-boundaries: shearing box BCs  */
#if FLUX_REMAP
#else
	for(j=0;j<NY;j++)
	for(k=0;k<NZ;k++) {
		for(i=lo;i<=-1;i++) var[i][j][k] = var[i+NX][j][k] ;
		  var[NX][j][k] = var[0][j][k] ;
		var[NX+1][j][k] = var[1][j][k] ;
	}
#endif
	if(q!=0.) {
	del = -q*W*Lx*tcurr/dy ;
	for(k=0;k<NZ;k++) {
#if LINEAR_REMAP
		for(i=lo;i<=-1;i++) remap_linear(var,i,k,del) ;
                printf("del = %20.12g\n",del);
		   
                remap_linear(var,NX,k,-del) ;
		remap_linear(var,NX+1,k,-del) ;
#elif FOURIER_REMAP
		for(i=lo;i<=-1;i++) remap_fourier(var,i,k,del) ;
		
                remap_fourier(var,NX,k,del) ;
		remap_fourier(var,NX+1,k,del) ;
#elif FLUX_REMAP
		remap_flux(var,-1,k,NX,-del) ;

		remap_flux(var,-2,k,NX,-del) ;

		if(var==rho) remap_flux(var,-3,k,NX,-del) ;
                
		remap_flux(var,NX,k,-NX,del) ;

		remap_flux(var,NX+1,k,-NX,del) ;
#endif
	}
	}
	
	
	if(var==bx) {
	for(j=0;j<NY;j++)
	for(k=0;k<NZ;k++) {
		bx[NX][j][k] = bx[NX-1][j][k] + dx*(
			    + (by[NX-1][j][k] - by[NX-1][j+1][k])/dy
			    + (bz[NX-1][j][k] - bz[NX-1][j][k+1])/dz) ;
	}
	}
       
	
#if SWEEPYM
#else
        if(var==vy&&q!=0.) for(j=0;j<NY;j++)
	for(k=0;k<NZ;k++) {
                var[NX][j][k] += -q*W*Lx ;
                var[NX+1][j][k] += -q*W*Lx ;
                var[-1][j][k] += q*W*Lx ;
                var[-2][j][k] += q*W*Lx ;
        }
#endif

	/* y-boundaries: periodic */
	for(i=lo;i<NX+2;i++)
	for(k=0;k<NZ;k++) {
		for(j=lo;j<=-1;j++) var[i][j][k] = var[i][j+NY][k] ;
		var[i][NY][k] = var[i][0][k] ;
		var[i][NY+1][k] = var[i][1][k] ;
	}

        /* z-boundaries: periodic */
#if PERIODIC
	for(i=lo;i<NX+2;i++)
	for(j=lo;j<NY+2;j++) {
		for(k=lo;k<=-1;k++) var[i][j][k] = var[i][j][k+NZ] ;
		var[i][j][NZ] = var[i][j][0] ;
		var[i][j][NZ+1] = var[i][j][1] ;
	}
#endif

#if REFLECTING
	/* z-boundaries: reflecting, for symmetric variables rho,vx,vy */
	if(sym == SYMMETRIC) {
		for(i=lo;i<NX+2;i++)
		for(j=lo;j<NY+2;j++) {
			var[i][j][-1] = var[i][j][0] ;
			var[i][j][-2] = var[i][j][1] ;
			if(var==rho) var[i][j][-3] = var[i][j][2] ;
			var[i][j][NZ] = var[i][j][NZ-1] ;
			var[i][j][NZ+1] = var[i][j][NZ-2] ;
		}
	}
	else {	/* for antisymmetric variables, e.g. vz */
		for(i=0;i<=NX+1;i++)
		for(j=0;j<=NY+1;j++) {
			var[i][j][-2] = -var[i][j][2] ;
			var[i][j][-1] = -var[i][j][1] ;
			var[i][j][0] = 0. ;
			var[i][j][NZ] = 0. ;
			var[i][j][NZ+1] = -var[i][j][NZ-1] ;
		}
	}
#endif

#if OUTFLOW
#endif
}

void bound_var_trans(REAL (*var)[NY+5][NZ+5], REAL tcurr, int sym)
{
  	int i,j,k,lo ;
	REAL del ;
        REAL test;

	lo=-2;
	
		/* x-boundaries: shearing box BCs only for v1 */
	
	if(var == vx)
	  {
	    if(q!=0.) 
	      {
		del = -q*W*Lx*tcurr/dy ;
		for(k=0;k<NZ;k++) 
		  {
		    
		    remap_flux(var,-2,k,NX,-del) ;
		    
		  }
	      }
	  }
	
	
	// y-boundaries: periodic 
	for(i=lo;i<NX+2;i++)
	for(k=0;k<NZ;k++) 
	  {
	    for(j=lo;j<=-1;j++) var[i][j][k] = var[i][j+NY][k] ;
	    var[i][NY][k] = var[i][0][k] ;
	    var[i][NY+1][k] = var[i][1][k] ;
	  }

        // z-boundaries: periodic 
#if PERIODIC
	for(i=lo;i<NX+2;i++)
	  for(j=lo;j<NY+2;j++) 
	    {
	      for(k=lo;k<=-1;k++) 
		var[i][j][k] = var[i][j][k+NZ] ;
	    
	      var[i][j][NZ] = var[i][j][0] ;
	      var[i][j][NZ+1] = var[i][j][1] ;
	    }
#endif


	
}
