/***********************************************************************************
    Copyright 2011 Bryan Johnson, Xiaoyue Guan & Charles F. Gammie

                  mthreed  version 1.0   (released Jun 2, 2011)

    This file is part of mthreed. mthreed adopts a ZEUS-like scheme to solve 
    magnetohydrodynamic equations of motion in a shearing sheet. It uses obital 
    advection to ensure fast and accurate integration in a large shearing box. 
    

    You are morally obligated to cite the following paper in his/her
    scientific literature that results from use of any part of mthreed:

    [1] Johnson, B.M., Guan, X. \& Gammie, C.F. \ 2008, Astrophysical Journal, 
        Supplement, 177, 373


    The latest version of the mthreed is available on our code distribution
    website:
    http://rainman.astro.uiuc.edu/codelib/

    mthreed is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    mthreed is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mthreed; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

***********************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#define NX	128
#define NY	128
#define NZ	64

#define SMALL	1.e-10
#define GMIN 	1.e-10
#define MIN	1.e-20	/* minimum density */

#define REAL	double

#define LINEAR   	0  /* linear theory test */
#define HDMODE1  	0  /* epicyclic mode */
#define HDMODE2  	0  /* rotationally-modified sound waves */
#define MHDMODE1 	0  /* Alfven waves */
#define MHDMODE2 	0  /* magnetosonic waves */
#define MHDMODE3 	1  /* MRI */

#define VSHWAVE2D       0  /* 2D vortical shearing wave */
#define VSHWAVE         0  /* Vortical shearing wave */
#define CSHWAVE         0  /* Compressive shearing wave */
#define ISHWAVE         0  /* Incompressive shearing wave */
#define PLUS            0  /* Positive (1) or negative (0) branch of omega(k) */
#define ASHWAVE         0  /* Alfven shearing wave */
#define ADVECTB         0  /* Constant amplitude magnetic shearing wave */

#define RHOCOL		0  /* circular density column */
#define STRIPE		0  /* horizontal density stripe */
#define FLOOP		0  /* B-field loop */
#define BY0		0  /* linear time dependence of By */

#define SWEEPYM		1  /* mean-velocity advection */
#define LINEAR_REMAP	0  /* linear interpolation */
#define FLUX_REMAP	1  /* flux interpolation */
#define FOURIER_REMAP	0  /* fourier interpolation */
#define BVANLEER	1  /* use van Leer slopes for B-field shear advection */

extern REAL   rhoas[NX+5][NY+5][NZ+5] ;	
extern REAL   rhotmpas [NX+5][NY+5][NZ+5] ;
extern REAL   vxas[NX+5][NY+5][NZ+5] ;
extern REAL   vyas[NX+5][NY+5][NZ+5] ;
extern REAL   vzas[NX+5][NY+5][NZ+5] ;
extern REAL   pxas[NX+5][NY+5][NZ+5];
extern REAL   pyas[NX+5][NY+5][NZ+5];
extern REAL   pzas[NX+5][NY+5][NZ+5];


extern REAL   bxas[NX+5][NY+5][NZ+5] ;
extern REAL   byas[NX+5][NY+5][NZ+5] ;
extern REAL   bzas[NX+5][NY+5][NZ+5] ;
extern REAL   eas[NX+5][NY+5][NZ+5] ;
extern REAL   work1as[NX+4][NY+4][NZ+4] ;
extern REAL   work2as[NX+4][NY+4][NZ+4] ;
extern REAL   work3as[NX+4][NY+4][NZ+4] ;

extern REAL (* rho)[NY+5][NZ+5] ;
extern REAL (* rhotmp)[NY+5][NZ+5] ;
extern REAL (*  vx)[NY+5][NZ+5] ;
extern REAL (*  vy)[NY+5][NZ+5] ;
extern REAL (*  vz)[NY+5][NZ+5] ;
extern REAL (*  px)[NY+5][NZ+5] ;
extern REAL (*  py)[NY+5][NZ+5] ;
extern REAL (*  pz)[NY+5][NZ+5] ;

extern REAL (*  bx)[NY+5][NZ+5] ;
extern REAL (*  by)[NY+5][NZ+5] ;
extern REAL (*  bz)[NY+5][NZ+5] ;
extern REAL (*   e)[NY+5][NZ+5] ;
extern REAL (*  work1)[NY+4][NZ+4] ;
extern REAL (*  work2)[NY+4][NZ+4] ;
extern REAL (*  work3)[NY+4][NZ+4] ;

extern REAL cour ;
extern REAL cs ;
extern REAL c ;
extern REAL gam ;
extern REAL dx,dy,dz ;
extern REAL dt ;
extern REAL t,tf ;
extern REAL nu_vnr ;
extern REAL nu_l ;

extern REAL Lx ;
extern REAL Ly ;
extern REAL Lz ;

extern REAL dV ;

extern REAL DTd ;
extern REAL DTi ;
extern REAL DTl ;

extern REAL drive ;
extern REAL A0,B0,B0x,B0y,B0z ;
extern REAL va, vax, vay, vaz, vs, vadk, kappa2;
extern REAL work,workp ;
extern REAL W,Wz,q ;


extern REAL kx,ky,kz ;
extern REAL k0, k20, dkxdt;
extern REAL remr[8],remi[8],norm,pert ;
extern REAL F10,F20,F30,F40,dF1dt0,dF2dt0,d2F1dt20,d2F2dt20 ;

extern int nstep ;

void remap_linear(REAL (*var)[NY+5][NZ+5], int i, int k, REAL shift) ;
void remap_fourier(REAL (*var)[NY+5][NZ+5], int i, int k, REAL del) ;
void remap_flux(REAL (*var)[NY+5][NZ+5], int i, int k, int shift, REAL del) ;
void iremap(REAL *var, int shift) ;
int my_nint(double dum) ;
REAL slope_lim(double y1,double y2,double y3) ;
void bound_var(REAL (* var)[NY+5][NZ+5], REAL tcurr, int sym) ;
void bound_var_trans(REAL (*var)[NY+5][NZ+5], REAL tcurr, int sym);

#define LOOP for(i=0;i<NX;i++)for(j=0;j<NY;j++)for(k=0;k<NZ;k++)
#define LOOPP(a,b,c,d,e,f) for(i=0-(a);i<NX+(b);i++)for(j=0-(c);j<NY+(d);j++)for(k=0-(e);k<NZ+(f);k++)
/* find largest axis */
#define NN     ( (NX >= NY && NX >= NZ) ? NX : ( (NY >= NX && NY >= NZ) ? NY : NZ ) )

#define ANTISYMMETRIC	-1
#define SYMMETRIC	1

