/***********************************************************************************
    Copyright 2011 Bryan Johnson, Xiaoyue Guan & Charles F. Gammie

                  mthreed  version 1.0   (released Jun 2, 2011)

    This file is part of mthreed. mthreed adopts a ZEUS-like scheme to solve 
    magnetohydrodynamic equations of motion in a shearing sheet. It uses obital 
    advection to ensure fast and accurate integration in a large shearing box. 
    

    You are morally obligated to cite the following paper in his/her
    scientific literature that results from use of any part of mthreed:

    [1] Johnson, B.M., Guan, X. \& Gammie, C.F. \ 2008, Astrophysical Journal, 
        Supplement, 177, 373


    The latest version of the mthreed is available on our code distribution
    website:
    http://rainman.astro.uiuc.edu/codelib/

    mthreed is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    mthreed is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mthreed; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

***********************************************************************************/

#include "decs.h"

void remap_linear(REAL (*var)[NY+5][NZ+5], int i, int k, REAL del)
{
        int j,jp,jpp,idel ;
        REAL fshift, tmp[NY] ;

        /* separate into integral & fraction part of shift */
	while(del < 0.) del += NY ;
        idel = (int)floor(del) ;
        fshift = del - idel ;

	/* do integral part of shift */
	for(j=0;j<NY;j++) {
		jp = (j + idel)%NY ; /* gives non-fractional part of (j+idel)/NY */
		jpp = jp + 1 ;
		while(jpp > NY-1) jpp -= NY ;
		tmp[j] = (1. - fshift)*var[i][jp][k] + fshift*var[i][jpp][k] ;
	}
	for(j=0;j<NY;j++) var[i][j][k] = tmp[j] ;
}


void remap_fourier(REAL (*var)[NY+5][NZ+5], int i, int k, REAL del)
{
	REAL tmp[NY] ;
	REAL freq,tmpj,pfac,tfac = 2.*M_PI*del ;
	void drealft(double data[], int n, int isign);
	int j ;
	int Ny = NY ;

	for(j=0;j<NY;j++) tmp[j] = var[i][j][k] ;
	drealft(tmp-1,NY,1) ;

	for(j=2;j<NY;j+=2) {
		freq = ((REAL)j/(2.0*Ny)) ;
		pfac = freq*tfac ;
		tmp[j] = (tmpj=tmp[j])*cos(pfac) - tmp[j+1]*sin(pfac) ;
		tmp[j+1] = tmpj*sin(pfac) + tmp[j+1]*cos(pfac) ;
	}
	pfac = 0.5*tfac ;

	drealft(tmp-1,NY,-1) ;
	for(j=0;j<NY;j++) var[i][j][k] = 2.0*tmp[j]/Ny ;
}


void remap_flux(REAL (*var)[NY+5][NZ+5], int i, int k, int shift, REAL del)
{

     
	int j,jp,jm,idel,offset ;
	REAL fdel ;
	REAL dq[NY],flux[NY],mdot[NY],trnsvar[NY] ;
	REAL specvar[NY];
	void flux_calc(REAL *var, REAL *trnsvar, REAL *flux, int offset);
	void consistent_transport(REAL (*var)[NY+5][NZ+5], int i, int k,REAL *specvar, int fr);
	void flux_update(REAL *specvar, REAL *flux);
	

        /* separate into integral & fraction part of shift */
        idel = my_nint(del) ;
        fdel = del - (REAL)idel ;

	offset = 0;
	
	consistent_transport(var,i+shift,k,specvar,1) ;

	/* calculate  flux */
	for(j=0;j<NY;j++) 
	  trnsvar[j] = fdel ;
	flux_calc(specvar,trnsvar,flux,offset) ;


	/* update transport variable */
	flux_update(specvar,flux) ;

	/* do integral part of shift */
        iremap(specvar,idel) ;


	consistent_transport(var,i,k,specvar,-1) ;
}


void iremap(REAL *var, int idel)
{
	static REAL tmp[NY] ;
	int j,jp ;

	for(j=0;j<NY;j++) tmp[j] = var[j] ;

        for(j=0;j<NY;j++) {
                jp = j + idel ;
                while(jp < 0) jp += NY ;
                while(jp > NY-1) jp -= NY ;
                var[jp] = tmp[j] ;
	}
}


void consistent_transport(REAL (*var)[NY+5][NZ+5], int i, int k,REAL *specvar, int fr)
{
	int j,jm ;
	REAL rhoa[NY];

	if(fr==1) 
	  { 
	    
	    if(var == py)
	      {
		
		for(j=1;j<NY;j++)
		  rhoa[j] = 0.5*(rho[i][j][k] + rho[i][j-1][k]);
		rhoa[0] = 0.5*(rho[i][0][k] + rho[i][NY-1][k]);

		for(j=0;j<NY;j++)
		  {
#if SWEEPYM
                    specvar[j] = var[i][j][k];
#else

		    if((i==NX-1 )||(i==NX-2))
		      specvar[j] = var[i][j][k]+ q*W*Lx*rhoa[j];
		    
                    if((i==0 )||(i==1))
		      specvar[j] = var[i][j][k]- q*W*Lx*rhoa[j];
#endif
		  }
	      }
	    
	    else
	      
	      for(j=0;j<NY;j++) 
		{
		  specvar[j] = var[i][j][k];	
		}
	  }

	else if(fr==-1) 
	  {
	    for(j=0;j<NY;j++) var[i][j][k] = specvar[j] ;
	  }
	
	else {fprintf(stderr,"fr != 1 || -1\n") ; exit(0) ;}
}


void flux_calc(REAL *var, REAL *trnsvar, REAL *flux, int offset)
{
	int j,jm,jp ;
	REAL dq[NY] ;

	for(j=0;j<NY;j++) 
	  {
	    jp = j + 1 ;
	    while(jp > NY-1) 
	      jp -= NY ;

	    jm = j - 1 ;
	    while(jm < 0)    
	      jm += NY ;
	    dq[j] = slope_lim(var[jm],var[j],var[jp]) ;
	  }

	for(j=0;j<NY;j++) 
	  {
	    jp = j + offset ;
	    while(jp > NY-1) jp -= NY ;
	    jm = j - 1 ;
	    while(jm < 0)    jm += NY ;
	    flux[j] = (trnsvar[j] > 0.) ?
			trnsvar[j]*(var[jm] + (1. - trnsvar[j])*dq[jm]) :
			trnsvar[j]*(var[jp] - (1. + trnsvar[j])*dq[jp]) ;
	}
}


void flux_update(REAL *specvar, REAL *flux)
{
	int j,jm,jp ;

	for(j=0;j<NY;j++) {
		jp = j + 1 ;
		while(jp > NY-1) jp -= NY ;
		specvar[j] += -(flux[jp]-flux[j]) ;
	}
}


REAL slope_lim(REAL y1,REAL y2,REAL y3)
{
        REAL Dqm,Dqp,s ;

        /* van leer slope limiter */
	/* N.B.: factor of 2 is missing because factor is
	   absorbed into expressions for fluxes above */
	Dqp = (y3 - y2) ;
	Dqm = (y2 - y1) ;
	s = Dqm*Dqp ;
	if(s <= 0.) return 0. ;
	else return(s/(Dqm+Dqp)) ;

}


int my_nint(REAL z)
{
        if(z > 0) return((int)(z+0.5)) ;
        else return((int)(z-0.5)) ;
}


void bxremap(int i, int k, int idel, int fcase, REAL (*w)[NX])
{
	REAL tmp[NY],(*bx_dqy)[NY+4][NZ+4] ;
	int j,js,jc ;

#if BVANLEER
	bx_dqy = work1 ;
#endif
	for(j=0;j<NY;j++) {
		js = (j - idel)%NY ;
		while(js < 0) js += NY ;
		while(js > NY - 1) js -= NY ;
		tmp[j] = bx[i][js][k] ;
		switch(fcase) {
		case 1: jc = js - 1 ;
				while(jc < 0) jc += NY ;
				break ;
		case 2: jc = js - 1 ;
				while(jc < 0) jc += NY ;
				break ;
		case 3: jc = js + 1 ;
				while(jc > NY - 1) jc -= NY ;
				break ;
		}
		tmp[j] += w[0][i]*(bx[i][jc][k] - bx[i][js][k]) ;
#if BVANLEER
		tmp[j] += w[1][i]*(bx_dqy[i][jc][k] - bx_dqy[i][js][k]) ;
#endif
	}
	for(j=0;j<NY;j++) bx[i][j][k] = tmp[j] ;
}


void byremap(int i, int k, int idel, int fcase, REAL (*w)[NX])
{
	int j,js,jm,jp ;
	REAL tmp[NY],(*bx_dqy)[NY+4][NZ+4],(*bz_dqx)[NY+4][NZ+4],(*bz_dqy)[NY+4][NZ+4] ;
	
#if BVANLEER
	bx_dqy = work1 ;
	bz_dqx = work2 ;
	bz_dqy = work3 ;
#endif
	for(j=0;j<NY;j++) {
		js = (j - idel)%NY ;
		while(js < 0) js += NY ;
		while(js > NY - 1) js -= NY ;
		jm = js - 1 ;
		while(jm < 0) jm += NY ;
		jp = js + 1 ;
		while(jp > NY - 1) jp -= NY ;
		switch(fcase) {
		case 1: 
			if(nstep%2==0) tmp[j] = 0.5*(by[i][js][k] + by[i][jm][k]
							+ (dy/dx)*(bx[i][jm][k] - bx[i+1][jm][k])
							+ (dy/dz)*(bz[i][jm][k] - bz[i][jm][k+1])) ;
			else tmp[j] = 0.5*(by[i][js][k] + by[i][jp][k]
				  + (dy/dx)*(bx[i+1][js][k] - bx[i][js][k])
				  + (dy/dz)*(bz[i][js][k+1] - bz[i][js][k])) ;
			tmp[j] += (dy/dx)*(-w[0][i]*bx[i][jm][k] - w[2][i]*bx[i+1][js][k])
					+ (dy/dz)*(w[4][i]*(bz[i][jm][k+1] - bz[i][jm][k])
					+ w[7][i]*(bz[i][js][k] - bz[i][js][k+1])) ;
#if BVANLEER
			tmp[j] += (dy/dx)*(-w[1][i]*bx_dqy[i][jm][k] - w[3][i]*bx_dqy[i+1][js][k])
					+ (dy/dz)*(w[5][i]*(bz_dqx[i][jm][k+1] - bz_dqx[i][jm][k])
					+ w[6][i]*(bz_dqy[i][jm][k+1] - bz_dqy[i][jm][k])
					+ w[8][i]*(bz_dqx[i][js][k] - bz_dqx[i][js][k+1])
					+ w[9][i]*(bz_dqy[i][js][k] - bz_dqy[i][js][k+1])) ;
#endif
			break ;
		case 2: 
			tmp[j] = 0.5*(by[i][js][k] + by[i][jm][k]
					+ (dy/dx)*(bx[i][jm][k] - bx[i+1][jm][k])
					+ (dy/dz)*(bz[i][jm][k] - bz[i][jm][k+1])) ;
			tmp[j] += (dy/dx)*(-w[0][i]*bx[i][jm][k] + w[2][i]*bx[i+1][jm][k])
					+ (dy/dz)*w[4][i]*(bz[i][jm][k+1] - bz[i][jm][k]) ;
#if BVANLEER
			tmp[j] += (dy/dx)*(-w[1][i]*bx_dqy[i][jm][k] + w[3][i]*bx_dqy[i+1][jm][k])
					+ (dy/dz)*(w[5][i]*(bz_dqx[i][jm][k+1] - bz_dqx[i][jm][k])
					+ w[6][i]*(bz_dqy[i][jm][k+1] - bz_dqy[i][jm][k])) ;
#endif
			break ;
		case 3: 
			tmp[j] = 0.5*(by[i][js][k] + by[i][jp][k]
					+ (dy/dx)*(bx[i+1][js][k] - bx[i][js][k])
					+ (dy/dz)*(bz[i][js][k+1] - bz[i][js][k])) ;
			tmp[j] += (dy/dx)*(w[0][i]*bx[i][js][k] - w[2][i]*bx[i+1][js][k])
					+ (dy/dz)*w[4][i]*(bz[i][js][k] - bz[i][js][k+1]) ;
#if BVANLEER
			tmp[j] += (dy/dx)*(w[1][i]*bx_dqy[i][js][k] - w[3][i]*bx_dqy[i+1][js][k])
					+ (dy/dz)*(w[5][i]*(bz_dqx[i][js][k] - bz_dqx[i][js][k+1])
					+ w[6][i]*(bz_dqy[i][js][k] - bz_dqy[i][js][k+1])) ;
#endif
			break ;
		}
	}
	for(j=0;j<NY;j++) by[i][j][k] = tmp[j] ;
}


void bzremap(int i, int k, int idel, int fcase, REAL (*w)[NX])
{
	int j,js,jm,jp ;
	REAL tmp[NY],(*bz_dqx)[NY+4][NZ+4],(*bz_dqy)[NY+4][NZ+4] ;

#if BVANLEER
	bz_dqx = work2 ;
	bz_dqy = work3 ;
#endif

	for(j=0;j<NY;j++) {
		js = (j - idel)%NY ;
		while(js < 0) js += NY ;
		while(js > NY - 1) js -= NY ;
		tmp[j] = bz[i][js][k] ;
		switch(fcase) {
		case 1:
			jp = js + 1 ;
			while(jp > NY - 1) jp -= NY ;
			jm = js - 1 ;
			while(jm < 0) jm += NY ;
			tmp[j] += w[4][i]*(bz[i][jm][k] - bz[i][js][k])
			        + w[7][i]*(bz[i][jp][k] - bz[i][js][k]) ;
#if BVANLEER
			tmp[j] += w[5][i]*(bz_dqx[i][jm][k] - bz_dqx[i][js][k])
					+ w[6][i]*(bz_dqy[i][jm][k] - bz_dqy[i][js][k])
					+ w[8][i]*(bz_dqx[i][jp][k] - bz_dqx[i][js][k])
					+ w[9][i]*(bz_dqy[i][jp][k] - bz_dqy[i][js][k]) ;
#endif
			break ;
		case 2:
			jm = js - 1 ;
			while(jm < 0) jm += NY ;
			tmp[j] += w[4][i]*(bz[i][jm][k] - bz[i][js][k]) ;
#if BVANLEER
			tmp[j] += w[5][i]*(bz_dqx[i][jm][k] - bz_dqx[i][js][k])
					+ w[6][i]*(bz_dqy[i][jm][k] - bz_dqy[i][js][k]) ;
#endif
			break ;
		case 3:
			jp = js + 1 ;
			while(jp > NY - 1) jp -= NY ;
			tmp[j] += w[4][i]*(bz[i][jp][k] - bz[i][js][k]) ;
#if BVANLEER
			tmp[j] += w[5][i]*(bz_dqx[i][jp][k] - bz_dqx[i][js][k])
					+ w[6][i]*(bz_dqy[i][jp][k] - bz_dqy[i][js][k]) ;
#endif
			break ;
		}
	}
	for(j=0;j<NY;j++) bz[i][j][k] = tmp[j] ;
}


void weights(int i, int fcase, REAL f, REAL sh, REAL (*w)[NX])
{
	REAL nm,np,cnm,cnp ;
	int n ;

	nm = sh/2. + f ;
	np = sh/2. - f ;
	cnm = nm*nm/sh ;
	cnp = np*np/sh ;

	switch(fcase) {
	case 1: w[0][i] = nm ;
			w[1][i] = nm*(1. - nm) ;
			w[2][i] = np ;
			w[3][i] = np*(np - 1.) ;
			w[4][i] = cnm/2. ;
			w[5][i] = cnm*(f - sh)/(3.*sh) ;
			w[6][i] = cnm*(1.5 - nm)/3. ;
			w[7][i] = cnp/2. ;
			w[8][i] = cnp*(f + sh)/(3.*sh) ;
			w[9][i] = cnp*(np - 1.5)/3. ;
			break ;
	case 2: w[0][i] = nm ;
			w[1][i] = nm*(1. - nm) ;
			w[2][i] = -np ;
			w[3][i] = -np*(np + 1.) ;
			w[4][i] = f ;
			w[5][i] = -sh/6. ;
			w[6][i] = f*(1. - f) - sh*sh/12. ;
			break ;
	case 3: w[0][i] = -nm ;
			w[1][i] = nm*(1. + nm) ;
			w[2][i] = np ;
			w[3][i] = np*(np - 1.) ;
			w[4][i] = -f ;
			w[5][i] = sh/6. ;
			w[6][i] = f*(1. + f) + sh*sh/12. ;
			break ;
	}
}

