/***********************************************************************************
    Copyright 2011 Bryan Johnson, Xiaoyue Guan & Charles F. Gammie

                  mthreed  version 1.0   (released Jun 2, 2011)

    This file is part of mthreed. mthreed adopts a ZEUS-like scheme to solve 
    magnetohydrodynamic equations of motion in a shearing sheet. It uses obital 
    advection to ensure fast and accurate integration in a large shearing box. 
    

    You are morally obligated to cite the following paper in his/her
    scientific literature that results from use of any part of mthreed:

    [1] Johnson, B.M., Guan, X. \& Gammie, C.F. \ 2008, Astrophysical Journal, 
        Supplement, 177, 373


    The latest version of the mthreed is available on our code distribution
    website:
    http://rainman.astro.uiuc.edu/codelib/

    mthreed is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    mthreed is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mthreed; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

***********************************************************************************/

/* 
 * Function sweepy
 * 
 * Transport step; sweep in y direction
 * 
 * bound_var 5 time: rho, vx, vy, vz, e
 *
 *
 * mpi version last modified: xg Mar 30, 2007
 * 
 *
 */
#include "decs.h"



void sweepy_new2c()
{
	REAL vpp,mdotp ;
	int i,j,k ;
	REAL (*dq)[NY+4][NZ+4], (*mdot)[NY+4][NZ+4], (*fl)[NY+4][NZ+4] ;
	
	dq = work1 ;
	mdot = work2 ;
	fl = work3 ;


	/* create normalized transport variable */
	

	LOOPP(2,2,2,2,2,2) 
	  vy[i][j][k] *= dt/dy ;
	
	/* first do mass */
	LOOPP(1,0,2,1,1,0) 
	  {
	    dq[i][j][k] = slope_lim(rho[i][j-1][k],rho[i][j][k],rho[i][j+1][k]) ;
	  }
	LOOPP(1,0,1,1,1,0) 
	  {
	    mdot[i][j][k] = (vy[i][j][k] > 0.) ?
	      vy[i][j][k]*(rho[i][j-1][k] + (1. - vy[i][j][k])*dq[i][j-1][k]) :
	      vy[i][j][k]*(rho[i][j][k] - (1. + vy[i][j][k])*dq[i][j][k]) ;

	  }


	LOOPP(0,0,1,1,0,0) 
	{
	  dq[i][j][k] = slope_lim(e[i][j-1][k],
				  e[i][j][k],
				  e[i][j+1][k]) ;
	  }
	LOOPP(0,0,0,1,0,0) 
	  {
	    fl[i][j][k] = (vy[i][j][k] > 0.) ?
	      vy[i][j][k]*(e[i][j-1][k] + (1. - vy[i][j][k])*dq[i][j-1][k]) :
	      vy[i][j][k]*(e[i][j][k] - (1. + vy[i][j][k])*dq[i][j][k]) ;
	  }
	LOOP 
	  e[i][j][k] += -(fl[i][j+1][k]-fl[i][j][k]) ;
	
	/* vx: because of different centering, indices differ */
	LOOPP(0,0,1,1,0,0) 
	  {
	    dq[i][j][k] = slope_lim(px[i][j-1][k],px[i][j][k],px[i][j+1][k]) ;
	  }
	LOOPP(0,0,0,1,0,0) 
	  {
	    vpp = 0.5*(vy[i][j][k]+vy[i-1][j][k]) ;
	    fl[i][j][k] = (vpp > 0.) ?
			vpp*(px[i][j-1][k] + (1. - vpp)*dq[i][j-1][k]) :
			vpp*(px[i][j][k] - (1. + vpp)*dq[i][j][k]) ;
	  }
	LOOP 
	  px[i][j][k] += -(fl[i][j+1][k]-fl[i][j][k]) ;
	
	/* vy */
	LOOPP(0,0,1,1,0,0) 
	  {
	    dq[i][j][k] = slope_lim(py[i][j-1][k],py[i][j][k],py[i][j+1][k]) ;
	  }
	LOOPP(0,0,1,0,0,0) 
	  {
	    vpp = 0.5*(vy[i][j][k]+vy[i][j+1][k]) ;
	    fl[i][j][k] = (vpp > 0.) ?
			vpp*(py[i][j][k] + (1. - vpp)*dq[i][j][k]) :
			vpp*(py[i][j+1][k] - (1. + vpp)*dq[i][j+1][k]) ;
	  }
	LOOP 
	  py[i][j][k] += -(fl[i][j][k]-fl[i][j-1][k]) ;

	/* vz */
	LOOPP(0,0,1,1,0,0) 
	  {
	    dq[i][j][k] = slope_lim(pz[i][j-1][k],pz[i][j][k],pz[i][j+1][k]) ;
	  }
	LOOPP(0,0,0,1,0,0) 
	  {
	    vpp = 0.5*(vy[i][j][k]+vy[i][j][k-1]) ;
	    fl[i][j][k] = (vpp > 0.) ?
	      vpp*(pz[i][j-1][k] + (1. - vpp)*dq[i][j-1][k]) :
	      vpp*(pz[i][j][k] - (1. + vpp)*dq[i][j][k]) ;
	  }
	LOOP 
	  pz[i][j][k] += -(fl[i][j+1][k]-fl[i][j][k]) ;

	LOOP 
	  {
	    rho[i][j][k] += -(mdot[i][j+1][k]-mdot[i][j][k]) ;
	    if(rho[i][j][k] < MIN) {
	      fprintf(stderr,"y small density: %d %g\n",i,rho[i][j][k]) ;
	    }
	  }

	LOOPP(2,2,2,2,2,2) vy[i][j][k] *= dy/dt ;
		
	bound_var(rho,t+dt,SYMMETRIC) ;
	bound_var(e,t+dt,SYMMETRIC) ;
	
	 bound_var(px,t+dt,SYMMETRIC) ;
        bound_var(py,t+dt,SYMMETRIC) ;
        bound_var(pz,t+dt,ANTISYMMETRIC) ;

	
}


