/***********************************************************************************
    Copyright 2011 Bryan Johnson, Xiaoyue Guan & Charles F. Gammie

                  mthreed  version 1.0   (released Jun 2, 2011)

    This file is part of mthreed. mthreed adopts a ZEUS-like scheme to solve 
    magnetohydrodynamic equations of motion in a shearing sheet. It uses obital 
    advection to ensure fast and accurate integration in a large shearing box. 
    

    You are morally obligated to cite the following paper in his/her
    scientific literature that results from use of any part of mthreed:

    [1] Johnson, B.M., Guan, X. \& Gammie, C.F. \ 2008, Astrophysical Journal, 
        Supplement, 177, 373


    The latest version of the mthreed is available on our code distribution
    website:
    http://rainman.astro.uiuc.edu/codelib/

    mthreed is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    mthreed is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mthreed; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

***********************************************************************************/

/* 
 * Function sweepym
 * 
 * Transport step; orbital advection in y direction
 * 
 * bound_var 7 time: rho, e, vx, vy, vz, by, bz, bx
 *
 *
 * mpi version last modified: xg Apr 11, 2007
 * notes:
 *
 */
#include "decs.h"

void sweepym()
{
	REAL X,XP,del,sh,f,vxave ;
	int i,j,k,idel,fcase ;
	void bxremap(int i, int k, int idel, int fcase, REAL (*w)[NX]) ;
	void byremap(int i, int k, int idel, int fcase, REAL (*w)[NX]) ;
	void bzremap(int i, int k, int idel, int fcase, REAL (*w)[NX]) ;
	void weights(int i, int fcase, REAL f, REAL sh, REAL (*w)[NX]) ;
	void bound_var(REAL (*var)[NY+5][NZ+5], REAL tcurr, int sym) ;
	REAL (*bx_dqy)[NY+4][NZ+4],(*bz_dqx)[NY+4][NZ+4],(*bz_dqy)[NY+4][NZ+4] ;
	static REAL was[10][NX] ;
	static REAL (*w)[NX] ;


	w = (REAL(*)[NX])(&(was[0][0])) ;
#if BVANLEER
	bx_dqy = work1 ;
	bz_dqx = work2 ;
	bz_dqy = work3 ;
	
       
        LOOPP(0,1,0,0,0,0)
	  {
	    bx_dqy[i][j][k] = slope_lim(bx[i][j-1][k],bx[i][j][k],bx[i][j+1][k]) ;
	  }
	  LOOPP(0,0,0,0,0,1) 
	  {
		bz_dqx[i][j][k] = slope_lim(bz[i-1][j][k],bz[i][j][k],bz[i+1][j][k]) ;
		bz_dqy[i][j][k] = slope_lim(bz[i][j-1][k],bz[i][j][k],bz[i][j+1][k]) ;
	}
#endif
	sh = q*W*(dx/dy)*dt ;

	/* shift and evolve b_y, preserving div.b = 0  */
	/* must do this first, because byremap assumes that
	 *  bx,bz still take on their old values 
	 */

	for(i=0;i<NX;i++)
	  {
	    X = (i + 0.5)*dx - 0.5*Lx ;

	    /* cell shift = (length shift)*(cells/length) 
             *  or Delta cells = (Delta y)/dy = v0*(Delta t)/dy 
             */
	    del = -q*W*X*dt/dy ; 
	    idel = my_nint(del) ;
	    f = del - idel ;
	    if(fabs(f) <= sh/2.) fcase = 1 ;
	    else if(f > sh/2.) fcase = 2 ;
	    else fcase = 3 ;
	    weights(i,fcase,f,sh,w) ; /* calculate weights for all three remaps here */
	    for(k=0;k<NZ;k++) byremap(i,k,idel,fcase,w) ;
	}
	
	for(i=0;i<NX;i++)
	  {
	    XP = i*dx - 0.5*Lx ;


	    del = -q*W*XP*dt/dy ;
	    for(k=0;k<NZ;k++) 
	      {
#if LINEAR_REMAP
		remap_linear(px,i,k,-del) ;
#elif FOURIER_REMAP
		remap_fourier(px,i,k,del) ;
#elif FLUX_REMAP
		remap_flux(px,i,k,0,del) ;
#endif
	      }
	  }

	for(i=0;i<NX;i++) 
	  {
	    X = (i + 0.5)*dx - 0.5*Lx ;
	    
	    del = -q*W*X*dt/dy ;
	    idel = my_nint(del) ;
	    f = del - idel ;
	    if(fabs(f) <= sh/2.) fcase = 1 ;
	    else if(f > sh/2.) fcase = 2 ;
	    else fcase = 3 ;
	    for(k=0;k<NZ;k++) 
	      {
		bxremap(i,k,idel,fcase,w) ;
		bzremap(i,k,idel,fcase,w) ;
#if LINEAR_REMAP
		remap_linear(rho,i,k,-del) ;
		remap_linear(e,i,k,-del) ;
		remap_linear(py,i,k,-del) ;
		remap_linear(pz,i,k,-del) ;
#elif FOURIER_REMAP
		remap_fourier(rho,i,k,del) ;
		remap_fourier(e,i,k,del) ;
		remap_fourier(py,i,k,del) ;
		remap_fourier(pz,i,k,del) ;
#elif FLUX_REMAP
		remap_flux(e,i,k,0,del) ;
		remap_flux(py,i,k,0,del) ;
		remap_flux(pz,i,k,0,del) ;
		remap_flux(rho,i,k,0,del) ; /* need to update rho last */
#endif
	      }
	  }




	bound_var(rho,t+dt,SYMMETRIC) ; /* need to bound rho first */
	bound_var(e,t+dt,SYMMETRIC) ;
/*
	 if(W!=0.) LOOP {
                get_global_ik( i, k, &i_glob, &k_glob);
                XP = i_glob*dx - 0.5*Lx ;
                vx[i][j][k] -= (dt)*2.*q*W*W*XP ;
        }
*/
        bound_var(px,t+dt,SYMMETRIC) ;
 /* 
       if(W!=0.) LOOP {
                vxave = 0.25*(vx[i][j][k] + vx[i+1][j][k] +
                        vx[i][j-1][k] + vx[i+1][j-1][k]) ;

                vy[i][j][k] += dt*q*W*vxave ;

       }
*/

	bound_var(py,t+dt,SYMMETRIC) ;
	bound_var(pz,t+dt,ANTISYMMETRIC) ;
	bound_var(by,t+dt,SYMMETRIC) ;
	bound_var(bz,t+dt,ANTISYMMETRIC) ;
        bound_var(bx,t+dt,SYMMETRIC) ;
}

